// -*- C++ -*-
/*!
 * @file  CameraImageConvert.cpp
 * @brief CameraImageConvert
 * @date $Date$
 *
 * @author メディアドライブ
 * rtc-dev@mediadrive.jp
 *
 * BSD 3-clause "New" or "Revised" License
 *
 * $Id$
 */

#include "CameraImageConvert.h"
#include "ImageDataConvert.h"

// Module specification
// <rtc-template block="module_spec">
static const char* cameraimageconvert_spec[] =
  {
    "implementation_id", "CameraImageConvert",
    "type_name",         "CameraImageConvert",
    "description",       "CameraImageConvert",
    "version",           "1.0.0",
    "vendor",            "MEDIA DRIVE",
    "category",          "ImageProcessing",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.compress", "jpeg",
    "conf.default.quality", "95",

    // Widget
    "conf.__widget__.compress", "radio",
    "conf.__widget__.quality", "spin",
    // Constraints
    "conf.__constraints__.compress", "(off,jpeg)",
    "conf.__constraints__.quality", "0<=x<=100",

    "conf.__type__.compress", "string",
    "conf.__type__.quality", "int",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
CameraImageConvert::CameraImageConvert(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_RTCCameraImageInIn("RTCCameraImageIn", m_RTCCameraImageIn),
    m_RTCCameraImageOutOut("RTCCameraImageOut", m_RTCCameraImageOut)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
CameraImageConvert::~CameraImageConvert()
{
}


/*!
 * 各変数の初期化を行います
 */
RTC::ReturnCode_t CameraImageConvert::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("RTCCameraImageIn", m_RTCCameraImageInIn);
  
  // Set OutPort buffer
  addOutPort("RTCCameraImageOut", m_RTCCameraImageOutOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("compress", m_compress, "jpeg");
  bindParameter("quality", m_quality, "95");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CameraImageConvert::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageConvert::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageConvert::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageConvert::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*!
 * 確保した画像メモリを開放します
 */

RTC::ReturnCode_t CameraImageConvert::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*!
 * InPortから受け取った画像をコンフィギュレーション・パラメータに合
 * わせて圧縮、復元しOutPortへ送ります
 */

RTC::ReturnCode_t CameraImageConvert::onExecute(RTC::UniqueId ec_id)
{
    static coil::TimeValue tm_pre;
    static int count = 0;

  if(m_RTCCameraImageInIn.isNew())
  {
	  m_RTCCameraImageInIn.read();
    cv::Mat img = GetMatImage(m_RTCCameraImageIn);
    SetCameraImage(m_RTCCameraImageOut, img, m_compress, m_quality);
    m_RTCCameraImageOutOut.write();

    if (count > 100)
    {
        count = 0;
        coil::TimeValue tm;
        tm = coil::gettimeofday();
        double sec(tm - tm_pre);

        if (sec > 1.0 && sec < 1000.0)
        {
            std::cout << 100.0/sec << " [FPS]" << std::endl;
        }
        tm_pre = tm;
    }

    ++count;

  }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t CameraImageConvert::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageConvert::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageConvert::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageConvert::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t CameraImageConvert::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void CameraImageConvertInit(RTC::Manager* manager)
  {
    coil::Properties profile(cameraimageconvert_spec);
    manager->registerFactory(profile,
                             RTC::Create<CameraImageConvert>,
                             RTC::Delete<CameraImageConvert>);
  }
  
};


